<?php
/**
 * Main WPSyncSheetsGravityForms namespace.
 *
 * @since 1.0.0
 * @package wpsyncsheets-gravity-forms
 */

use WPSyncSheetsGravityForms\WPSSLGF_Google_API_Functions;
/**
 * Handle plugin installation upon activation.
 *
 * @since 1.0.0
 */
class WPSSLGF_Plugin_Settings {

	/**
	 * Plugin documentation URL
	 *
	 * @var $documentation
	 */
	protected static $documentation = 'https://docs.wpsyncsheets.com/wpssg-introduction/';
	/**
	 * Plugin support URL
	 *
	 * @var $submit_ticket
	 */
	protected static $submit_ticket = 'https://wordpress.org/support/plugin/wpsyncsheets-gravity-forms/';
	/**
	 * Plugin sheet setting URL
	 *
	 * @var $doc_sheet_setting
	 */
	protected static $doc_sheet_setting = 'https://docs.wpsyncsheets.com/wpssg-google-sheets-api-settings/';
	/**
	 * WPSyncSheets For Gravity Forms Pro Plugin URL
	 *
	 * @var $pro_version_url
	 */
	protected static $pro_version_url = 'https://www.wpsyncsheets.com/wpsyncsheets-for-gravity-forms/';
	/**
	 * Instance of WPSSLGF_Plugin_Settings
	 *
	 * @var $instance
	 */
	private static $instance = null;

	/**
	 * Instance of WPSSLGF_Feed_Settings
	 *
	 * @var $instanceaddon
	 */
	private static $instanceaddon = null;

	/**
	 * Instance of WPSSLGF_Google_API_Functions
	 *
	 * @var $instance_api
	 */
	private static $instance_api = null;

	/**
	 * Primary class constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		// When activated, trigger install method.
		add_action( 'admin_menu', array( $this, 'wpsslgf_menu_page' ), 40 );
		add_action( 'admin_enqueue_scripts', array( $this, 'wpsslgf_load_custom_wp_admin_style' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'wpsslgf_selectively_enqueue_admin_script' ) );
		add_filter( 'gform_noconflict_scripts', array( $this, 'wpsslgf_register_gform_noconflict_script' ) );
		add_filter( 'gform_noconflict_styles', array( $this, 'wpsslgf_register_gform_noconflict_style' ) );
		add_filter( 'plugin_row_meta', array( $this, 'wpsslgf_plugin_row_meta' ), 10, 2 );
		add_action( 'gform_loaded', array( $this, 'wpsslgf_load' ) );
		$this->wpsslgf_google_api();
	}

	/**
	 * Main WPSSLGF_Plugin_Settings Instance.
	 *
	 * @since 1.0.0
	 *
	 * @return instance
	 */
	public static function instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Register GGSFeedAddOn Addon
	 */
	public function wpsslgf_load() {
		if ( ! method_exists( 'GFForms', 'include_feed_addon_framework' ) ) {
			return;
		}
		GFAddOn::register( 'WPSSLGF_Feed_Settings' );
	}

	/**
	 * Create WPSSLGF_Feed_Settings Class Instance.
	 */
	public static function get_addoninstance() {
		if ( null === self::$instanceaddon ) {
			self::$instanceaddon = new \WPSSLGF_Feed_Settings();
		}
		return self::$instanceaddon;
	}

	/**
	 * Create Google Api Instance.
	 */
	public static function wpsslgf_google_api() {

		if ( null === self::$instance_api ) {
			self::$instance_api = new WPSSLGF_Google_API_Functions();
		}
		return self::$instance_api;
	}
	/**
	 * Register a plugin menu page.
	 */
	public function wpsslgf_menu_page() {
		global $admin_page_hooks, $_parent_pages;
		if ( ! isset( $admin_page_hooks['wpsyncsheets_lite'] ) ) {
			$wpsslgf_page = add_menu_page(
				esc_attr__( 'WPSyncSheets Lite', 'wpssg' ),
				'WPSyncSheets Lite',
				'manage_options',
				'wpsyncsheets_lite',
				'',
				WPSSLGF_URL . 'assets/images/menu-icon.svg',
				90
			);
		}
		add_submenu_page( 'wpsyncsheets_lite', 'WPSyncSheets Lite For Gravity Forms', 'For Gravity Forms', 'manage_options', 'wpsyncsheets-gravity-forms', array( $this, 'wpsslgf_plugin_page' ), 1 );
		if ( ! isset( $_parent_pages['documentation'] ) ) {
			add_submenu_page( 'wpsyncsheets_lite', 'Documentation', '<div class="wpsslgf-support">Documentation</div>', 'manage_options', 'documentation', array( $this, 'wppsg_handle_external_redirects' ), 20 );
		}
		$this->remove_duplicate_submenu_page();

	}

	/**
	 * Documentation and Support Page Link.
	 *
	 * Redirect the documentation and support page.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function wppsg_handle_external_redirects() {
		// phpcs:ignore
		if ( empty( $_GET['page'] ) ) {
			return;
		}

		// phpcs:ignore
		if ( 'documentation' === $_GET['page'] ) {
			// phpcs:ignore
			wp_redirect( WPSSLGF_DOC_MENU_URL );
			die;
		}
	}

	/**
	 * Remove duplicate submenu
	 * Submenu page hack: Remove the duplicate WPSyncSheets Plugin link on subpages
	 */
	public function remove_duplicate_submenu_page() {
		remove_submenu_page( 'wpsyncsheets_lite', 'wpsyncsheets_lite' );
	}

	/**
	 * Load Required Javascript and Css.
	 */
	public static function wpsslgf_load_custom_wp_admin_style() {
		if ( isset( $_GET['page'] ) && ( 'wpsyncsheets-gravity-forms' === sanitize_text_field( wp_unslash( $_GET['page'] ) ) ) ){ // phpcs:ignore

			wp_register_script( 'wpsslgf-wp-admin', WPSSLGF_URL . 'assets/js/wpsslgf-admin-script.js', array(), WPSSLGF_VERSION, true );
			wp_localize_script(
				'wpsslgf-wp-admin',
				'admin_ajax_object',
				array(
					'ajaxurl'          => admin_url( 'admin-ajax.php' ),
					'sync_nonce_token' => wp_create_nonce( 'sync_nonce' ),
				)
			);
			wp_enqueue_script( 'wpsslgf-wp-admin' );
			wp_register_style( 'wpsslgf-wp-admin', WPSSLGF_URL . 'assets/css/wpsslgf-admin-style.css', false, WPSSLGF_VERSION );
			wp_enqueue_style( 'wpsslgf-wp-admin' );
		}
		// phpcs:ignore
		if ( isset( $_GET['page'] ) && ( 'gf_edit_forms' === sanitize_text_field( wp_unslash( $_GET['page'] ) ) ) && isset( $_GET['subview'] ) && 'ggspreadsheetaddon' === sanitize_text_field( ( $_GET['subview'] ) ) ) {
			wp_register_style( 'wpsslgf-feed-pro', WPSSLGF_URL . 'assets/css/wpsslgf-feed-pro.css', false, WPSSLGF_VERSION );
			wp_enqueue_style( 'wpsslgf-feed-pro' );
		}
	}

	/**
	 * Enqueue css and js files
	 */
	public static function wpsslgf_selectively_enqueue_admin_script() {
		wp_enqueue_script( 'wpsslgf-general-script', WPSSLGF_URL . 'assets/js/wpsslgf-general.js', WPSSLGF_VERSION, true, false );
	}

	/**
	 * Register Gravity Forms no conflict Script
	 *
	 * @param array $scripts Enqueue Scripts.
	 */
	public static function wpsslgf_register_gform_noconflict_script( $scripts ) {
		$scripts[] = 'wpsslgf-wp-admin';
		$scripts[] = 'wpsslgf-general-script';
		return $scripts;
	}

	/**
	 * Register Gravity Forms no conflict Styles
	 *
	 * @param array $styles Enqueue Styles.
	 */
	public static function wpsslgf_register_gform_noconflict_style( $styles ) {
		$styles[] = 'wpsslgf-wp-admin';
		$styles[] = 'wpsslgf-feed-pro';
		return $styles;
	}

	/**
	 * Show row meta on the plugin screen.
	 *
	 * @param mixed $wpsslgf_links Plugin Row Meta.
	 * @param mixed $wpsslgf_file  Plugin Base file.
	 * @return  array
	 */
	public static function wpsslgf_plugin_row_meta( $wpsslgf_links, $wpsslgf_file ) {

		if ( 'wpsyncsheets-gravity-forms/wpsyncsheets-lite-gravity-forms.php' === (string) $wpsslgf_file ) {
			$wpsslgf_row_meta = array(
				'docs' => '<a href="' . esc_url( self::$documentation ) . '" title="' . esc_attr( __( 'View Documentation', 'wpssg' ) ) . '" target="_blank">' . __( 'View Documentation', 'wpssg' ) . '</a>',
			);
			return array_merge( $wpsslgf_links, $wpsslgf_row_meta );
		}
		return (array) $wpsslgf_links;
	}

	/**
	 * Create Plugin Page.
	 */
	public function wpsslgf_plugin_page() {

		$wpsslgf_error = '';
		// Google API Settings Tab.
		if ( isset( $_POST['submit'] ) ) {

			if ( ! isset( $_POST['wpsslgf_api_settings'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['wpsslgf_api_settings'] ) ), 'save_api_settings' ) ) {
				$wpsslgf_error = '<strong class="err-msg">' . esc_html__( 'Error: Sorry, your nonce did not verify.', 'wpssg' ) . '</strong>';
			} else {
				if ( isset( $_POST['client_token'] ) ) {
					$wpsslgf_clienttoken = sanitize_text_field( wp_unslash( $_POST['client_token'] ) );
				} else {
					$wpsslgf_clienttoken = '';
				}
				if ( isset( $_POST['client_id'] ) && isset( $_POST['client_secret'] ) ) {
					$wpsslgf_google_settings = array( sanitize_text_field( wp_unslash( $_POST['client_id'] ) ), sanitize_text_field( wp_unslash( $_POST['client_secret'] ) ), $wpsslgf_clienttoken );
				} else {
					$wpsslgf_google_settings_value = self::$instance_api->wpsslgf_option( 'wpssg_google_settings' );
					$wpsslgf_google_settings       = array( $wpsslgf_google_settings_value[0], $wpsslgf_google_settings_value[1], $wpsslgf_clienttoken );
				}

				self::$instance_api->wpsslgf_update_option( 'wpssg_google_settings', $wpsslgf_google_settings );
			}
		}
		if ( isset( $_POST['revoke'] ) ) {
			if ( ! isset( $_POST['wpsslgf_api_settings'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['wpsslgf_api_settings'] ) ), 'save_api_settings' ) ) {
				$wpsslgf_error = '<strong class="err-msg">' . esc_html__( 'Error: Sorry, your nonce did not verify.', 'wpssg' ) . '</strong>';
			} else {
				$wpsslgf_google_settings    = self::$instance_api->wpsslgf_option( 'wpssg_google_settings' );
				$wpsslgf_google_settings[2] = '';
				self::$instance_api->wpsslgf_update_option( 'wpssg_google_settings', $wpsslgf_google_settings );
				self::$instance_api->wpsslgf_update_option( 'wpssg_google_accessToken', '' );
			}
		}

		$wpsslgf_google_settings_value = self::$instance_api->wpsslgf_option( 'wpssg_google_settings' );
		if ( isset( $wpsslgf_google_settings_value[2] ) && ! empty( $wpsslgf_google_settings_value[2] ) ) {
			if ( ! self::$instance_api->getClient() || 'invalid_grant' === self::$instance_api->getClient( 1 ) ) {
				$wpsslgf_error = self::$instance_api->getClient( 1 );
				if ( 'Invalid token format' === (string) $wpsslgf_error || 'invalid_grant' === (string) $wpsslgf_error ) {
					$wpsslgf_error = '<div class="eror token_error"><p><strong class="err-msg">' . esc_html__( 'Error: Invalid Token - Revoke Token with below settings and try again.', 'wpssg' ) . '</strong></p></div>';
				} else {
					$wpsslgf_error = '<div class="error token_error"><p><strong class="err-msg">Error: ' . $wpsslgf_error . '</strong></p></div>';
				}
			}
		}
		?>
		<div class="vertical-tabs">
			<div class="wpsslgf-logo-section">
				<img src="<?php echo esc_url( WPSSLGF_URL ); ?>assets/images/logo.png">
					<sup>V<?php echo esc_html( WPSSLGF_VERSION ); ?></sup>				
					<div class="duc-btn1 pro-version">
						<a target="_blank" href="<?php echo esc_url( self::$pro_version_url ); ?>">Pro Version</a>
					</div>
					<div class="duc-btn1 ">
						<a target="_blank" href="<?php echo esc_url( self::$submit_ticket ); ?>"><?php echo esc_html__( 'Support', 'wpssg' ); ?></a>
					</div>
					<div class="duc-btn">
						<a target="_blank" href='<?php echo esc_url( self::$documentation ); ?>' ><?php echo esc_html__( 'Documentation', 'wpssg' ); ?></a>
					</div>
			</div>
			<div class="tab">
				<button class="tablinks googleapi-settings" onclick="wpsslgftab(event, 'googleapi-settings')">
					<span class="tab-icon"></span>
					<?php echo esc_html__( 'Google API', 'wpssg' ); ?> <br><?php echo esc_html__( 'Settings', 'wpssg' ); ?>
				</button>
				<button class="tablinks support" onclick="wpsslgftab(event, 'support')">
					<span class="tab-icon"></span>
					<?php echo esc_html__( 'Pro Features', 'wpssg' ); ?>
				</button>
			</div>
		<div id="googleapi-settings" class="tabcontent">
			<h3><?php echo esc_html__( 'Google API Settings', 'wpssg' ); ?></h3>
			<p><?php echo esc_html__( 'Create new google APIs with Client ID and Client Secret keys to get an access for the google drive and google sheets. Please follow the documentation, login to your Gmail Account and start with', 'wpssg' ); ?> <a href="<?php echo esc_url( self::$doc_sheet_setting ); ?>" target="_blank ">here.</a></p>
			<form method="post" action="<?php echo esc_html( admin_url( 'admin.php?page=wpsyncsheets-gravity-forms' ) ); ?>">
				<?php wp_nonce_field( 'save_api_settings', 'wpsslgf_api_settings' ); ?>
				<div id="universal-message-container woocommerce">
				<br>
					<div class="options">
						<?php
						if ( ! empty( $wpsslgf_error ) ) {
							$allowed_html = wp_kses_allowed_html( 'post' );
							echo wp_kses( $wpsslgf_error, $allowed_html );
						}
						?>
					</div>
					<?php
					if ( isset( $wpsslgf_google_settings_value[0] ) ) {
						$client_id = $wpsslgf_google_settings_value[0];
					} else {
						$client_id = '';
					}
					if ( isset( $wpsslgf_google_settings_value[1] ) ) {
						$client_secret = $wpsslgf_google_settings_value[1];
					} else {
						$client_secret = '';
					}
					?>
						<table class="form-table">
						<tr>
							<th><?php echo esc_html__( 'Client Id', 'wpssg' ); ?> </th>
							<td class="forminp forminp-text">
								<input type="text" name="client_id" value="<?php echo esc_attr( $client_id ); ?>" size="80" class = "googlesettinginput" placeholder="<?php echo esc_html__( 'Enter Client Id', 'wpssg' ); ?>" required
								<?php
								if ( ! empty( $wpsslgf_google_settings_value[0] ) ) {
									echo 'readonly';
								}
								?>
								/>
							</td>
						</tr>
						<tr>
							<th><?php echo esc_html__( 'Client Secret', 'wpssg' ); ?> </th>
							<td class="forminp forminp-text">
								<input type="text" name="client_secret" value="<?php echo esc_attr( $client_secret ); ?>" size="80" class = "googlesettinginput" placeholder="<?php echo esc_html__( 'Enter Client Secret', 'wpssg' ); ?>" required
								<?php
								if ( ! empty( $wpsslgf_google_settings_value[1] ) ) {
									echo 'readonly';
								}
								?>
								/>
							</td>
						</tr>
						<?php
						if ( ! empty( $wpsslgf_google_settings_value[0] ) && ! empty( $wpsslgf_google_settings_value[1] ) ) {

								$token_value = $wpsslgf_google_settings_value[2];

							?>
						<tr>
							<th><?php echo esc_html__( 'Client Token', 'wpssg' ); ?></th>
							<?php
							if ( empty( $token_value ) && ! isset( $_GET['code'] ) ) {
								$auth_url = self::$instance_api->getClient();
								?>
							<td id="authbtn">
								<a href="<?php echo esc_url( $auth_url ); ?>" target="_blank" id="authlink"><div class="wpsslgf-button wpsslgf-button-secondary"><?php echo esc_html__( 'Click here to generate an Authentication Token', 'wpssg' ); ?></div></a>
							</td>
								<?php
							}
							$wpsslgf_code = '';
							if ( isset( $_GET['code'] ) && ! empty( sanitize_text_field( wp_unslash( $_GET['code'] ) ) ) ) {
								$wpsslgf_code = sanitize_text_field( wp_unslash( $_GET['code'] ) );
							}
							?>
							<td id="authtext" 
							<?php
							if ( ! empty( $token_value ) || $wpsslgf_code ) {
								?>
								class = "forminp forminp-text wpsslgf-authtext" 
								<?php
							} else {
								?>
								class="forminp forminp-text" 
								<?php
							}
							?>
							><input type="text" name="client_token" value="<?php echo $token_value ? esc_attr( $token_value ) : esc_attr( $wpsslgf_code ); ?>" size="80" placeholder="<?php echo esc_html__( 'Please enter authentication code', 'wpssg' ); ?>" id="client_token" class="googlesettinginput"
							<?php
							if ( ! empty( $wpsslgf_google_settings_value[2] ) ) {
								echo 'readonly';
							}
							?>
							/></td>
						</tr>
							<?php
						}
						if ( ! empty( $token_value ) ) {
							?>
						<tr>
							<td></td>
							<td><input type="submit" name="revoke" id="revoke" value = "Revoke Token" class="wpsslgf-button wpsslgf-button-secondary"/></td>
						</tr>
						<?php } ?>    
						</table>
					</div>
					<p class="submit">
						<input type="submit" name="submit" id="submit" class="wpsslgf-button wpsslgf-button-primary" value="Save">
						<?php
						if ( ! empty( $token_value ) || ! empty( $wpsslgf_google_settings_value[0] ) || ! empty( $wpsslgf_google_settings_value[1] ) ) {
							?>
							<input type="submit" name="reset_settings" id="reset_settings" value = "<?php echo esc_attr__( 'Reset Settings', 'wpssg' ); ?>" class="wpsslgf-button wpsslgf-button-primary reset_settings"/>
						<?php } ?>
					</p>
					<?php
					if ( isset( $_SERVER['SERVER_NAME'] ) ) {
						$site_url = sanitize_text_field( wp_unslash( $_SERVER['SERVER_NAME'] ) );
						$site_url = str_replace( 'www.', '', $site_url );
					}
					?>
					<table class="copy-url-table" cellpadding="0" cellspacing="0" width="100%" border="0px">
						<tr>
							<td><?php echo esc_html__( 'Authorized Domain : ', 'wpssg' ); ?></td>
							<td><span id="authorized_domain"><?php echo esc_html( $site_url ); ?></span><span class="copy-icon wpsslgf-button wpsslgf-button-primary" id="a_domain" onclick="wpsslgf_copy('authorized_domain','a_domain');"></span></td>
						</tr>
						<tr>
							<td><?php echo esc_html__( 'Authorised redirect URIs : ', 'wpssg' ); ?></td>
							<td><span id="authorized_uri"><?php echo esc_html( admin_url( 'admin.php?page=wpsyncsheets-gravity-forms' ) ); ?></span><span class="copy-icon wpsslgf-button wpsslgf-button-primary" onclick="wpsslgf_copy('authorized_uri','a_uri');" id="a_uri"></span></td>
						</tr>
						</table>
			</form>
		</div>             
		<div id="support" class="tabcontent">
				<a href="<?php echo esc_url( self::$pro_version_url ); ?>" target="_blank"><img src="<?php echo esc_url( plugin_dir_url( __DIR__ ) . 'assets/images/pro-features.png' ); ?>" /></a>
		</div> 
	</div>
		<?php
	}
}
new WPSSLGF_Plugin_Settings();
